<?php
/**********************************************************************************************
 * author         : Ad Bakker
 * plugin         : newMemScr
 * last_mod       : 07.24.2015 (July 24th 2015)
 * version        : 2.0
 *  copyright    : GNU General Public License version 3 ot later
 *********************************************************************************************
 * Plugin for screening new members. Methods:
 * 
 * 1-st number of posts must be approved by a moderator (number to be defined in settings)
 * 
 * Moderators get a message (e-mail, PM or none) when a new post or thread is added to
 * the moderation queue.
 * 
 * New members whose post must be moderated get a message (e-mail, PM or none) when their post
 * has been approved.
 *
 * Sumscribers to threads in which new a post is approved get a (delayed) message about this. 
 * The type of message is equal to that set in the subscription.
 *
 * Subscribers to forums in which a new thread is approved get a (delayed) message about this.
 * As with all forum subscriptions this is an e-mail
 *********************************************************************************************/

function newMemScr_info()
{
	return array(
		'name'			=> 'New member screening',
		'description'	=> 'Screen new users by requiring approval of their first post(s), and send appropriate messages to moderators, these new members and subscribers',
		'website'		=> '',
		'author'		=> 'Ad Bakker',
        'authorsite'    => '',
		'version'		=> '2.0',
		'guid'        	=> '',
        'compatibility' => '18*',
	);
}

function newMemScr_install()
{
	global $db, $lang;
/*
 * settinggroup : newMemScr
 * contains all settings for this plugin
 */

	$lang->load('newMemScr');
	$newMemScr_group = array(
		"gid"			=> "NULL",
		"name"			=> "newMemScr",
		"title"         => $db->escape_string($lang->newMemScr_setgroup_title),
		"description"	=> "Settings for the New Member Screening plugin.",
		"disporder"		=> "1234567890",
		"isdefault"		=> "no",
	);
	$db->insert_query("settinggroups", $newMemScr_group);
	$gid = $db->insert_id();
/*
 * setting : newMemScr_num_app
 * sets the number of posts of a new member that have to be approved before free posting
 */
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_num_app', 
		'title'			=> $db->escape_string($lang->newMemScr_numapp_title),
		'description'	=> $db->escape_string($lang->newMemScr_numapp_desc),
		'optionscode'	=> 'text',
		'value'			=> '1',
		'disporder'		=> '1',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);
/*
 * setting : newMemScr_forums
 * sets the forums in which the approval system for new members is active
 */
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_forums',
		'title'			=> $db->escape_string($lang->newMemScr_forums_title),
		'description'	=> $db->escape_string($lang->newMemScr_forums_desc),
		'optionscode'	=> 'forumselect',
		'value'			=> '',
		'disporder'		=> '2',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);
/*
 * setting : newMemScr_mod_message
 * Type of message to be sent to moderators (0 = no message, 1 = e-mail, 2 = PM )
 */
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_mod_message',
		'title'			=> $db->escape_string($lang->newMemScr_mod_message_title),
		'description'	=> $db->escape_string($lang->newMemScr_mod_message_desc),
		'optionscode'	=> 'select\n0=No\n1=E-mail\n2=PM',
		'value'			=> 1,
		'disporder'		=> '3',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);
/*
 * setting : newMemScr_additional_modgroups
 * Additional usergroups with the necessary moderator permissions to approve threads and posts
 *
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_additional_modgroups',
		'title'			=> $db->escape_string($lang->newMemScr_additional_modgroups_title),
		'description'	=> $db->escape_string($lang->newMemScr_additional_modgroups_desc),
		'optionscode'	=> 'groupselect',
		'value'			=> '',
		'disporder'		=> '4',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);
/*
 * setting : newMemScr_approved_message
 * Type of message to be sent to authors when their post/thread has been approved
 * (0 = no message, 1 = e-mail, 2 = PM )
 */
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_approved_message',
		'title'			=> $db->escape_string($lang->newMemScr_approved_message_title),
		'description'	=> $db->escape_string($lang->newMemScr_approved_message_desc),
		'optionscode'	=> 'select\n0=No\n1=E-mail\n2=PM',
		'value'			=> 1,
		'disporder'		=> '5',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);
/*
 * setting : newMemScr_subscribe_message
 * Indicates whether subscribers to a thread or forum should be sent a message when a
 * post or thread has been approved (0 = n0, 1 = yes)
 */
	$newMemScr_setting = array(
		"sid"			=> "NULL",
		'name'			=> 'newMemScr_subscribe_message',
		'title'			=> $db->escape_string($lang->newMemScr_subscribe_message_title),
		'description'	=> $db->escape_string($lang->newMemScr_subscribe_message_desc),
		'optionscode'	=> 'yesno',
		'value'			=> 1,
		'disporder'		=> '6',
		'gid'			=> $gid,
	);
	$db->insert_query('settings', $newMemScr_setting);

	rebuild_settings();
}

function newMemScr_uninstall()
{
	global $db;
	$db->delete_query("settinggroups","name='newMemScr'");
	$db->query("DELETE FROM ".TABLE_PREFIX."settings WHERE name LIKE 'newMemScr_%'");
}

function newMemScr_is_installed()
{
    global $db;
    $query = $db->simple_select("settinggroups", "*", "name='newMemScr'");
    if($db->num_rows($query))
    {
        return true;
    }
    return false;
}

/**********************************************************************************************
 * Plugin function that decides whether a new reply or a new thread must be moderared
 *********************************************************************************************/
$plugins->add_hook('newthread_do_newthread_start', 'newMemScr_mod_decision');
$plugins->add_hook('newreply_do_newreply_start', 'newMemScr_mod_decision');
function newMemScr_mod_decision()
{
	global $mybb, $fid;
	if($mybb->settings['newMemScr_num_app'] > $mybb->user['postnum'] && (in_array($fid, explode(',', $mybb->settings['newMemScr_forums'])) || $mybb->settings['newMemScr_forums'] == "-1"))
	{
		$mybb->user['moderateposts'] = 1;
	}
}

/**********************************************************************************************
 * Plugin function that sends moderators a message about a new reply that requires moderation
 *********************************************************************************************/
$plugins->add_hook("newreply_do_newreply_end", "newMemScr_modmessage_post");
function newMemScr_modmessage_post ($posthandler)
{
    global $db, $mybb, $visible, $lang, $cache, $fid, $forum, $tid, $thread, $pid, $post;

    if ($visible != 0 || $mybb->settings['newMemScr_mod_message'] == 0) return; 
    $lang->load("newMemScr");
	$langcache = array();
	$args = array(
		'poster'   => $post['username'],
		'thread'   => $thread['subject'],
		'forum'    => $forum['name'],
		'link' => get_post_link($pid, $thread['tid'])."#pid".$pid
	);
	$excerpt = my_substr($post['message'], 0, $mybb->settings['subscribeexcerpt']);
	$where = newMemScr_mod_where($fid);
	$query = $db->simple_select("users u", "u.uid, u.username, u.email, u.language", $db->escape_string($where));
	while($mod = $db->fetch_array($query))
	{
		newMemScr_send_message ($mod, 'newMemScr_mod_message_post_message', 'newMemScr_mod_message_post_subject', $mybb->settings['newMemScr_mod_message'], $excerpt, $args);
	}
	if ($mybb->settings['newMemScr_mod_message'] == 1) $cache->update_mailqueue();
}

/**********************************************************************************************
 * Plugin function that sends moderators a message about a new thread that requires moderation
 *********************************************************************************************/
$plugins->add_hook("newthread_do_newthread_end", "newMemScr_modmessage_thread");
function newMemScr_modmessage_thread ($posthandler)
{
global $db, $mybb, $visible, $lang, $cache, $forum, $new_thread, $tid;
	
    global $db, $mybb, $visible, $lang, $cache, $fid, $forum, $tid, $new_thread;
	$thread = $new_thread;
    if ($visible != 0 || $mybb->settings['newMemScr_mod_message'] == 0) return; 
    $lang->load("newMemScr");
	$args = array(
		'poster'   => $thread['username'],
		'thread'   => $thread['subject'],
		'forum'    => $forum['name'],
		'link' => get_thread_link($tid)
	);
	$excerpt = my_substr($thread['message'], 0, $mybb->settings['subscribeexcerpt']);
	$where = newMemScr_mod_where($fid);
	$query = $db->simple_select("users u", "u.uid, u.username, u.email, u.language", $db->escape_string($where));
	while($mod = $db->fetch_array($query))
	{
		newMemScr_send_message ($mod, 'newMemScr_mod_message_thread_message', 'newMemScr_mod_message_thread_subject', $mybb->settings['newMemScr_mod_message'], $excerpt, $args);
	}
	if ($mybb->settings['newMemScr_mod_message'] == 1) $cache->update_mailqueue();
}

/**********************************************************************************************
 * Plugin function that sends authors a message when their new post has been approved
 * Also sends messages to members who have subscribed to the thread of the new post
 *********************************************************************************************/
$plugins->add_hook("class_moderation_approve_posts", "newMemScr_approved_posts");
function newMemScr_approved_posts($pids)
{
	global $db, $mybb, $lang, $cache;
	if ($mybb->settings['newMemScr_approved_message'] == 0 || empty($pids)) return;
	$lang->load("newMemScr");
	$pids_string = implode(",", $pids);
	$query_app = $db->query("
		SELECT p.pid, p.uid, p.username, p.message, p.fid, p.tid, t.subject, f.name, u.email, u.language
		FROM ".TABLE_PREFIX."posts p
		LEFT JOIN ".TABLE_PREFIX."threads t ON (t.tid=p.tid)
		LEFT JOIN ".TABLE_PREFIX."forums f ON (f.fid=p.fid)
		LEFT JOIN ".TABLE_PREFIX."users u ON (u.uid=p.uid)
		WHERE p.pid IN ($pids_string)
	");
	while($post = $db->fetch_array($query_app))
	{
		$args = array(
			'poster'   => $post['username'],
			'thread'   => $post['subject'],
			'forum'    => $post['name'],
			'link'     => get_post_link($post['pid'], $post['tid'])."#pid".$post['pid'],
			'tid'      => $post['tid']
		);
		$excerpt = my_substr($post['message'], 0, $mybb->settings['subscribeexcerpt']);
		newMemScr_send_message ($post, 'newMemScr_author_message_post_message', 'newMemScr_author_message_post_subject', $mybb->settings['newMemScr_approved_message'], $excerpt, $args);
		if ($mybb->settings['newMemScr_subscribe_message'] == 0) continue; // no subscription messages required
		// find subscribers to this forum
		$query_subscr = $db->query("
			SELECT s.subscriptionkey, s.notification, u.uid, u.username, u.email, u.loginkey, u.salt, u.regdate
			FROM ".TABLE_PREFIX."threadsubscriptions s
			LEFT JOIN ".TABLE_PREFIX."users u ON (u.uid=s.uid)
			WHERE s.tid = {$post['tid']}
		");
		while ($subscriber = $db->fetch_array($query_subscr))
		{
			if ($subscriber['username'] == $post['username']) continue;
			$args['key'] = $subscriber['subscriptionkey'];
			$args['post_key'] = md5($subscriber['loginkey'].$subscriber['salt'].$subscriber['regdate']);
			newMemScr_send_message ($subscriber, 'newMemScr_threadsubscr_message', 'newMemScr_threadsubscr_subject', $subscriber['notification'], $excerpt, $args);
		}
	}
	$cache->update_mailqueue();
}

/**********************************************************************************************
 * Plugin function that sends authors a message when their new thread has been approved
 * Also sends messages to members who have subscribed to the forum of the new thread
 *********************************************************************************************/
$plugins->add_hook("class_moderation_approve_threads", "newMemScr_approved_threads");
function newMemScr_approved_threads($tids)
{
	global $db, $mybb, $lang, $cache, $forum;
	if ($mybb->settings['newMemScr_approved_message'] == 0 || empty($tids)) return;
	$lang->load("newMemScr");
	$tids_string = implode(",", $tids);
	$query_app = $db->query("
		SELECT p.pid, t.uid, t.username, p.message, t.fid, t.tid, t.subject, f.name, u.email, u.language
		FROM ".TABLE_PREFIX."threads t
		LEFT JOIN ".TABLE_PREFIX."posts p ON (p.pid = t.firstpost)
		LEFT JOIN ".TABLE_PREFIX."forums f ON (f.fid=t.fid)
		LEFT JOIN ".TABLE_PREFIX."users u ON (u.uid=t.uid)
		WHERE t.tid IN ($tids_string)
	");
	while($thread = $db->fetch_array($query_app))
	{
		$args = array(
			'poster'   => $thread['username'],
			'thread'   => $thread['subject'],
			'forum'    => $thread['name'],
			'link'     => get_thread_link($thread['tid']),
			'fid'      => $thread['fid']
		);
		$excerpt = my_substr($thread['message'], 0, $mybb->settings['subscribeexcerpt']);
		newMemScr_send_message ($thread, 'newMemScr_author_message_thread_message', 'newMemScr_author_message_thread_subject', $mybb->settings['newMemScr_approved_message'], $excerpt, $args);
		if ($mybb->settings['newMemScr_subscribe_message'] == 0) continue; // no subscription messages required
		// find subscribers to this forum
		$query_subscr = $db->query("
			SELECT u.uid, u.username, u.email, u.loginkey, u.salt, u.regdate
			FROM ".TABLE_PREFIX."forumsubscriptions s
			LEFT JOIN ".TABLE_PREFIX."users u ON (u.uid=s.uid)
			LEFT JOIN ".TABLE_PREFIX."usergroups g ON (g.gid=u.usergroup)
			WHERE s.fid = {$thread['fid']}
			AND s.uid != '".$thread['uid']."'
			AND u.lastactive > '{$forum['lastpost']}'
			AND g.isbannedgroup != 1
		");
		// Send subscribers a message that a new thread has been started
		while ($subscriber = $db->fetch_array($query_subscr))
		{
			$args['post_key'] = md5($subscriber['loginkey'].$subscriber['salt'].$subscriber['regdate']);
			newMemScr_send_message ($subscriber, 'newMemScr_forumsubscr_message', 'newMemScr_forumsubscr_subject', 1, $excerpt, $args);
		}
	}
	$cache->update_mailqueue();
}

/**********************************************************************************************
 * Function that sends a message (e-mail or PM) in the language of the addressee
 *********************************************************************************************/
function newMemScr_send_message ($addressee, $message_lvar, $subject_lvar, $message_type, $excerpt, $args)
{
	global $db, $mybb, $lang, $langcache;
	if($addressee['language'] != '' && $lang->language_exists($addressee['language']))
	{
		$uselang = $addressee['language'];
	}
	elseif($mybb->settings['orig_bblanguage'])
	{
		$uselang = $mybb->settings['orig_bblanguage'];
	}
	else
	{
		$uselang = "english";
	}
	if($uselang == $mybb->settings['bblanguage'])
	{
		$message = $lang->$message_lvar;
		$subject = $lang->$subject_lvar;
		$emailbit_viewthread = $lang->emailbit_viewthread;
	}
	else
	{
		if(!isset($langcache[$uselang]["{$subject_lvar}"]))
		{
			$userlang = new MyLanguage;
			$userlang->set_path(MYBB_ROOT."inc/languages");
			$userlang->set_language($uselang);
			$userlang->load("newMemScr");
			$userlang->load("messages");
// Below the complete contents of the language file newMemScr.lang.php is cached to avoid the necessity
// to make changes to function load in inc/class_language.php (change require_once into require)
			$langcache[$uselang]["newMemScr_mod_message_post_subject"] = $userlang->newMemScr_mod_message_post_subject;
			$langcache[$uselang]["newMemScr_mod_message_post_message"] = $userlang->newMemScr_mod_message_post_message;
			$langcache[$uselang]["newMemScr_mod_message_thread_subject"] = $userlang->newMemScr_mod_message_thread_subject;
			$langcache[$uselang]["newMemScr_mod_message_thread_message"] = $userlang->newMemScr_mod_message_thread_message;
			$langcache[$uselang]["newMemScr_author_message_post_subject"] = $userlang->newMemScr_author_message_post_subject;
			$langcache[$uselang]["newMemScr_author_message_post_message"] = $userlang->newMemScr_author_message_post_message;
			$langcache[$uselang]["newMemScr_author_message_thread_subject"] = $userlang->newMemScr_author_message_thread_subject;
			$langcache[$uselang]["newMemScr_author_message_thread_message"] = $userlang->newMemScr_author_message_thread_message;
			$langcache[$uselang]["newMemScr_threadsubscr_subject"] = $userlang->newMemScr_threadsubscr_subject;
			$langcache[$uselang]["newMemScr_threadsubscr_message"] = $userlang->newMemScr_threadsubscr_message;
			$langcache[$uselang]["newMemScr_forumsubscr_subject"] = $userlang->newMemScr_forumsubscr_subject;
			$langcache[$uselang]["newMemScr_forumsubscr_message"] = $userlang->newMemScr_forumsubscr_message;
			$langcache[$uselang]["emailbit_viewthread"] = $userlang->emailbit_viewthread;
			unset($userlang);
		}
		$subject = $langcache[$uselang]["{$subject_lvar}"];
		$message = $langcache[$uselang]["{$message_lvar}"];
		$emailbit_viewthread = $langcache[$uselang]["emailbit_viewthread"];
	}
	$args['addressee'] = $addressee['username'];
	$args['excerpt'] = $excerpt.$emailbit_viewthread;
	$args['boardname'] = $mybb->settings['bbname'];
	$args['bbhome'] = $mybb->settings['bburl'];
	$subject = sprintf_arr($subject, $args);
	$message = sprintf_arr($message, $args);
	if($message_type == 1)
	{
		$email = array(
			"mailto"   => $db->escape_string($addressee['email']),
			"mailfrom" => $db->escape_string($mybb->settings['returnemail']),
			"subject"  => $db->escape_string($subject),
			"message"  => $db->escape_string($message),
			"headers"  => ''
		);
		$db->insert_query("mailqueue", $email);
	}
	elseif ($message_type == 2)
	{
		$pm = array(
			'subject' => $subject,
			'touid'   => $addressee['uid'],
			'message' => $message
		);
		send_pm($pm, -1, true);
	}
}

/**********************************************************************************************
 * Function that constructs the where clause for finding the moderators of a forum
 *********************************************************************************************/
function newMemScr_mod_where($fid)
{
	global $db, $mybb;
	$where = "u.usergroup NOT IN (1,5,7) ";	
	// global moderators
	$where .= "AND (3 IN (u.usergroup,u.additionalgroups) ";
	// forum moderators
	$query = $db->simple_select("moderators", "id, isgroup", "fid = {$fid} AND canviewunapprove = 1 AND canapproveunapprovethreads = 1 AND canapproveunapproveposts = 1");
	while ($mod = $db->fetch_array($query))
	{
		if ($mod['isgroup'] == 0)
		{
			$where .= "OR u.uid = {$mod['id']} ";
		}
		else
		{
			$where .= "OR {$mod['id']} IN (u.usergroup,u.additionalgroups) ";
		}
	}
	$where .= ")";
	return $where;
}

/**********************************************************************************************
 * Function for the replacement of fields in language strings by an associative array
 * {$key} in the string is replaced by $value
 *********************************************************************************************/
function sprintf_arr($string, $args)
{
	foreach ($args as $key => $value)
	{
		$string = str_replace("{".$key."}", $value, $string);
	}
	return $string;
}
